@@ -28,8 +28,8 @@ static void transfer_relations(osm_t *osm, way_t *dst, way_t *src) {
     
   while(rchain) {
     relation_chain_t *next = rchain->next;
-    printf("way #%ld is part of relation #%ld\n", src->id, 
-	   rchain->relation->id);
+    printf("way #"ITEM_ID_FORMAT" is part of relation #"ITEM_ID_FORMAT"\n", 
+	   src->id, rchain->relation->id);
       
     /* make new member of the same relation */
     
@@ -38,15 +38,15 @@ static void transfer_relations(osm_t *osm, way_t *dst, way_t *src) {
     char *role = NULL;
     while(*member) { 
       /* save role of way */
-      if(((*member)->type == WAY) && ((*member)->way == src))
+      if(((*member)->object.type == WAY) && ((*member)->object.way == src))
 	role = (*member)->role;
       member = &(*member)->next;
     }
     
-    printf("  adding way #%ld to relation\n", dst->id);
+    printf("  adding way #"ITEM_ID_FORMAT" to relation\n", dst->id);
     *member = g_new0(member_t, 1);
-    (*member)->type = WAY;
-    (*member)->way = dst;
+    (*member)->object.type = WAY;
+    (*member)->object.way = dst;
     if(role) (*member)->role = g_strdup(role);
     member = &(*member)->next;
     
@@ -93,7 +93,8 @@ static gboolean combine_tags(tag_t **dst, tag_t *src) {
 /* -------------------------- way_add ----------------------- */
 
 void map_edit_way_add_begin(map_t *map, way_t *way_sel) {
-  if(way_sel) printf("previously selected way is #%ld\n", way_sel->id);
+  if(way_sel) printf("previously selected way is #"ITEM_ID_FORMAT"\n", 
+		     way_sel->id);
 
   g_assert(!map->action.way);
   map->action.way = osm_way_new();
@@ -123,7 +124,7 @@ void map_edit_way_add_segment(map_t *map, gint x, gint y) {
     /* use the existing node if one was touched */    
     node_t *node = map_hl_touchnode_get_node(map);
     if(node) {
-      printf("  re-using node #%ld\n", node->id);
+      printf("  re-using node #"ITEM_ID_FORMAT"\n", node->id);
       map_hl_touchnode_clear(map);
 
       g_assert(map->action.way);
@@ -135,12 +136,14 @@ void map_edit_way_add_segment(map_t *map, gint x, gint y) {
 	way_chain_t *next = way_chain->next;
 	
 	if(node == osm_way_get_last_node(way_chain->way)) {
-	  printf("  way #%ld ends with this node\n", way_chain->way->id);
+	  printf("  way #"ITEM_ID_FORMAT" ends with this node\n", 
+		 way_chain->way->id);
 	  if(!touch_way) touch_way = way_chain->way;
 	}
 	
 	if(node == osm_way_get_first_node(way_chain->way)) {
-	  printf("  way #%ld starts with this node\n", way_chain->way->id);
+	  printf("  way #"ITEM_ID_FORMAT" starts with this node\n", 
+		 way_chain->way->id);
 	  if(!touch_way) touch_way = way_chain->way;
 	}
 	
@@ -219,7 +222,7 @@ void map_edit_way_add_cancel(map_t *map) {
   while(chain) {
     node_chain_t *next = chain->next;
     
-    printf("    node #%ld (used by %d)\n", 
+    printf("    node #"ITEM_ID_FORMAT" (used by %d)\n", 
 	   chain->node->id, chain->node->ways);
     
     chain->node->ways--;
@@ -248,7 +251,7 @@ void map_edit_way_add_ok(map_t *map) {
   /* (their way count will be 0 after removing the way) */
   node_chain_t *chain = map->action.way->node_chain;
   while(chain) {
-    printf("    node #%ld (used by %d)\n", 
+    printf("    node #"ITEM_ID_FORMAT" (used by %d)\n", 
 	   chain->node->id, chain->node->ways);
     
     /* a node may have been a stand-alone node before, so remove its */
@@ -269,16 +272,17 @@ void map_edit_way_add_ok(map_t *map) {
   }
 
   /* attach to existing way if the user requested so */
-  gboolean revert = FALSE;
+  gboolean reverse = FALSE;
   if(map->action.extending) {
     node_t *nfirst = map->action.way->node_chain->node;
 
-    printf("  request to extend way #%ld\n", map->action.extending->id);
+    printf("  request to extend way #"ITEM_ID_FORMAT"\n", 
+	   map->action.extending->id);
 
     if(osm_way_get_first_node(map->action.extending) == nfirst) {
       printf("  need to prepend\n");
-      osm_way_revert(map->action.extending);
-      revert = TRUE;
+      osm_way_reverse(map->action.extending);
+      reverse = TRUE;
     } else if(osm_way_get_last_node(map->action.extending) == nfirst) 
       printf("  need to append\n");
     
@@ -302,8 +306,8 @@ void map_edit_way_add_ok(map_t *map) {
     map->action.way->flags |= OSM_FLAG_DIRTY;
 
     /* and undo reversion of required */
-    if(revert)
-      osm_way_revert(map->action.way);
+    if(reverse)
+      osm_way_reverse(map->action.way);
 
   } else {
     /* now move the way itself into the main data structure */
@@ -327,16 +331,16 @@ void map_edit_way_add_ok(map_t *map) {
   if(map->action.ends_on) {
     printf("  this new way ends on another way\n");
 
-    /* if revert is true the node in question is the first one */
-    /* of the newly created way. thus is it revertes again before */
-    /* attaching and the result is finally reverted once more */
+    /* If reverse is true the node in question is the first one */
+    /* of the newly created way. Thus it is reversed again before */
+    /* attaching and the result is finally reversed once more */
 
     /* this is slightly more complex as this time two full tagged */
     /* ways may be involved as the new way may be an extended existing */
     /* way being connected to another way. This happens if you connect */
     /* two existing ways using a new way between them */
     
-    if(revert) osm_way_revert(map->action.way);
+    if (reverse) osm_way_reverse(map->action.way);
 
     /* and open dialog to resolve tag collisions if necessary */
     if(combine_tags(&map->action.way->tag, map->action.ends_on->tag))
@@ -349,15 +353,15 @@ void map_edit_way_add_ok(map_t *map) {
     /* make way member of all relations ends_on already is */
     transfer_relations(map->appdata->osm, map->action.way, map->action.ends_on);
 
-    /* check if we have to revert (again?) to match the way order */
+    /* check if we have to reverse (again?) to match the way order */
     if(osm_way_get_last_node(map->action.ends_on) == 
        osm_way_get_last_node(map->action.way)) {
 
       printf("  need to prepend ends_on\n");
 
-      /* need to revert ends_on way */
-      osm_way_revert(map->action.ends_on);
-      revert = !revert;
+      /* need to reverse ends_on way */
+      osm_way_reverse(map->action.ends_on);
+      reverse = !reverse;
     }
 
     /* search end of node chain */
@@ -372,7 +376,7 @@ void map_edit_way_add_ok(map_t *map) {
     /* erase and free ends_on (now only containing the first node anymore) */
     map_way_delete(map->appdata, map->action.ends_on);
     
-    if(revert) osm_way_revert(map->action.way);
+    if(reverse) osm_way_reverse(map->action.way);
   }
 
   /* remove prior version of this way */
@@ -403,7 +407,7 @@ void map_edit_way_node_add_highlight(map_t *map, map_item_t *item,
 
 void map_edit_way_node_add(map_t *map, gint x, gint y) {
   /* check if we are still hovering above the selected way */
-  map_item_t *item = map_real_item_at(map, x, y);
+  map_item_t *item = map_item_at(map, x, y);
   if(item && map_item_is_selected_way(map, item)) {
     /* convert mouse position to canvas (world) position */
     canvas_window2world(map->canvas, x, y, &x, &y);
@@ -414,7 +418,7 @@ void map_edit_way_node_add(map_t *map, gint x, gint y) {
       osm_node_attach(map->appdata->osm, node);
       
       /* insert it into ways chain of nodes */
-      way_t *way = item->way;
+      way_t *way = item->object.way;
       
       /* search correct position */
       node_chain_t **chain = &way->node_chain;
@@ -465,19 +469,19 @@ void map_edit_way_cut_highlight(map_t *map, map_item_t *item, gint x, gint y) {
       gint x0, y0, x1, y1;
       canvas_item_get_segment_pos(item->item, seg, &x0, &y0, &x1, &y1);
       
-      gint width = (item->way->draw.flags & 
+      gint width = (item->object.way->draw.flags & 
 		    OSM_DRAW_FLAG_BG)?
-	2*item->way->draw.bg.width:
-	3*item->way->draw.width;
+	2*item->object.way->draw.bg.width:
+	3*item->object.way->draw.width;
       map_hl_segment_draw(map, width, x0, y0, x1, y1);
     }
   } else if(map_item_is_selected_node(map, item)) {
-    node_t *nfirst = osm_way_get_first_node(map->selected.way);
-    node_t *nlast = osm_way_get_last_node(map->selected.way);
+    node_t *nfirst = osm_way_get_first_node(map->selected.object.way);
+    node_t *nlast = osm_way_get_last_node(map->selected.object.way);
 
     /* cutting a way at its first or last node doesn't make much sense ... */
-    if((nfirst != item->node) && (nlast != item->node)) 
-      map_hl_cursor_draw(map, item->node->lpos.x, item->node->lpos.y, 
+    if((nfirst != item->object.node) && (nlast != item->object.node)) 
+      map_hl_cursor_draw(map, item->object.node->lpos.x, item->object.node->lpos.y, 
 			 TRUE, 2*map->style->node.radius);
   }
 }
@@ -486,7 +490,7 @@ void map_edit_way_cut_highlight(map_t *map, map_item_t *item, gint x, gint y) {
 void map_edit_way_cut(map_t *map, gint x, gint y) {
 
   /* check if we are still hovering above the selected way */
-  map_item_t *item = map_real_item_at(map, x, y);
+  map_item_t *item = map_item_at(map, x, y);
   if(item && (map_item_is_selected_way(map, item) || 
 	      map_item_is_selected_node(map, item))) {
     gboolean cut_at_node = map_item_is_selected_node(map, item);
@@ -500,15 +504,17 @@ void map_edit_way_cut(map_t *map, gint x, gint y) {
       printf("  cut at node\n");
       
       /* node must not be first or last node of way */
-      g_assert(map->selected.type == MAP_TYPE_WAY);
+      g_assert(map->selected.object.type == WAY);
       
-      if((osm_way_get_first_node(map->selected.way) != item->node) &&
-	 (osm_way_get_last_node(map->selected.way) != item->node)) {
-	way = map->selected.way;
+      if((osm_way_get_first_node(map->selected.object.way) != 
+	  item->object.node) &&
+	 (osm_way_get_last_node(map->selected.object.way) != 
+	  item->object.node)) {
+	way = map->selected.object.way;
 
 	cut_at = 0;
 	node_chain_t *chain = way->node_chain;
-	while(chain && chain->node != item->node) {
+	while(chain && chain->node != item->object.node) {
 	  chain = chain->next;
 	  cut_at++;
 	}
@@ -519,7 +525,7 @@ void map_edit_way_cut(map_t *map, gint x, gint y) {
     } else {
       printf("  cut at segment\n");
       cut_at = canvas_item_get_segment(item->item, x, y);
-      if(cut_at >= 0) way = item->way;
+      if(cut_at >= 0) way = item->object.way;
     }
     
     if(way) {
@@ -575,7 +581,7 @@ void map_edit_way_cut(map_t *map, gint x, gint y) {
       map_item_deselect(map->appdata);
       
       /* remove prior version of this way */
-      printf("remove visible version of way #%ld\n", way->id);
+      printf("remove visible version of way #"ITEM_ID_FORMAT"\n", way->id);
       map_item_chain_destroy(&way->map_item_chain);
       
       /* swap chains of the old way is to be destroyed due to a lack */
@@ -634,10 +640,10 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
   map_t *map = appdata->map;
   osm_t *osm = appdata->osm;
 
-  g_assert(map_item->type == MAP_TYPE_NODE);
-  node_t *node = map_item->node;
+  g_assert(map_item->object.type == NODE);
+  node_t *node = map_item->object.node;
 
-  printf("released dragged node #%ld\n", node->id);
+  printf("released dragged node #"ITEM_ID_FORMAT"\n", node->id);
   printf("  was at %d %d (%f %f)\n", 
 	 node->lpos.x, node->lpos.y,
 	 node->pos.lat, node->pos.lon);
@@ -650,7 +656,7 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
   if(touchnode) {
     map_hl_touchnode_clear(map);
 
-    printf("  dropped onto node #%ld\n", touchnode->id);
+    printf("  dropped onto node #"ITEM_ID_FORMAT"\n", touchnode->id);
 
     if(yes_no_f(GTK_WIDGET(appdata->window), 
 		appdata, MISC_AGAIN_ID_JOIN_NODES, 0,
@@ -671,7 +677,7 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 	node_chain_t *chain = way->node_chain;
 	while(chain) {
 	  if(chain->node == touchnode) {
-	    printf("  found node in way #%ld\n", way->id);
+	    printf("  found node in way #"ITEM_ID_FORMAT"\n", way->id);
 	    
 	    /* replace by node */
 	    chain->node = node;
@@ -692,11 +698,12 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
       while(relation) {
 	member_t *member = relation->member;
 	while(member) {
-	  if(member->type == NODE && member->node == touchnode) {
-	    printf("  found node in relation #%ld\n", relation->id);
+	  if(member->object.type == NODE && member->object.node == touchnode) {
+	    printf("  found node in relation #"ITEM_ID_FORMAT"\n", 
+		   relation->id);
 	    
 	    /* replace by node */
-	    member->node = node;
+	    member->object.node = node;
 
 	    relation->flags |= OSM_FLAG_DIRTY;
 	  }
@@ -735,7 +742,7 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 	  if(ways2join_cnt < 2) 
 	    ways2join[ways2join_cnt] = way;
 
-	  printf("  way #%ld ends with this node\n", way->id);
+	  printf("  way #"ITEM_ID_FORMAT" ends with this node\n", way->id);
 	  ways2join_cnt++;
 	}
 	way = way->next;
@@ -753,14 +760,14 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 		    _("Do you want to join the dragged way with the one "
 		      "you dropped it on?"))) {
 
-	  printf("  about to join ways #%ld and #%ld\n",
-		 ways2join[0]->id, ways2join[1]->id);
+	  printf("  about to join ways #" ITEM_ID_FORMAT " and #"
+		 ITEM_ID_FORMAT "\n", ways2join[0]->id, ways2join[1]->id);
 
 	  /* way[1] gets destroyed and attached to way[0] */
 	  /* so check if way[1] is selected and exchainge ways then */
 	  /* so that way may stay selected */
-	  if((map->selected.type == MAP_TYPE_WAY) && 
-	     (map->selected.way == ways2join[1])) {
+	  if((map->selected.object.type == WAY) && 
+	     (map->selected.object.way == ways2join[1])) {
 	    printf("  swapping ways to keep selected one alive\n");
 	    way_t *tmp = ways2join[1];
 	    ways2join[1] = ways2join[0];
@@ -769,12 +776,12 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 
 	  /* take all nodes from way[1] and append them to way[0] */
 	  /* check if we have to append or prepend to way[0] */
-	  gboolean revert = FALSE;
+	  gboolean reverse = FALSE;
 	  if(ways2join[0]->node_chain->node == node) {
-	    /* make "prepend" to be "append" by reverting way[0] */
-	    printf("  target prepend -> revert\n");	    
-	    revert = TRUE;
-	    osm_way_revert(ways2join[0]);
+	    /* make "prepend" to be "append" by reversing way[0] */
+	    printf("  target prepend -> reverse\n");	    
+	    reverse = TRUE;
+	    osm_way_reverse(ways2join[0]);
 	  } 
 	  
 	  /* verify the common node is last in the target way */
@@ -785,8 +792,8 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 	  
 	  /* common node must be first in the chain to attach */
 	  if(ways2join[1]->node_chain->node != node) {
-	    printf("  source revert\n");	    
-	    osm_way_revert(ways2join[1]);
+	    printf("  source reverse\n");	    
+	    osm_way_reverse(ways2join[1]);
 	  }
 	  
 	  /* verify the common node is first in the source way */
@@ -811,7 +818,8 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 
 	  while(rchain) {
 	    relation_chain_t *next = rchain->next;
-	    printf("way[1] is part of relation #%ld\n", rchain->relation->id);
+	    printf("way[1] is part of relation #"ITEM_ID_FORMAT"\n", 
+		   rchain->relation->id);
 
 	    /* make way[0] member of the same relation */
 
@@ -821,11 +829,12 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 	    member_t **member = &rchain->relation->member;
 	    char *role = NULL;
 	    while(*member && 
-		  !(((*member)->type == WAY) && 
-		    ((*member)->way == ways2join[0]))) {
+		  !(((*member)->object.type == WAY) && 
+		    ((*member)->object.way == ways2join[0]))) {
 
 	      /* save role of way[1] */
-	      if(((*member)->type == WAY) && ((*member)->way == ways2join[0]))
+	      if(((*member)->object.type == WAY) && 
+		 ((*member)->object.way == ways2join[0]))
 		role = (*member)->role;
 
 	      member = &(*member)->next;
@@ -836,8 +845,8 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
 	    else {
 	      printf("  adding way[0] to relation\n");
 	      *member = g_new0(member_t, 1);
-	      (*member)->type = WAY;
-	      (*member)->way = ways2join[0];
+	      (*member)->object.type = WAY;
+	      (*member)->object.way = ways2join[0];
 	      if(role) (*member)->role = g_strdup(role);
 	      member = &(*member)->next;
 
@@ -898,7 +907,7 @@ void map_edit_node_move(appdata_t *appdata, map_item_t *map_item,
   way_t *way = osm->way;
   while(way) {
     if(osm_node_in_way(way, node)) {
-      printf("  node is part of way #%ld, redraw!\n", way->id);
+      printf("  node is part of way #"ITEM_ID_FORMAT", redraw!\n", way->id);
       
       /* remove prior version of this way */
       map_item_chain_destroy(&way->map_item_chain);
@@ -928,11 +937,45 @@ void map_edit_way_reverse(appdata_t *appdata) {
   /* deleting the selected item de-selects it ... */
   map_item_deselect(appdata);
 
-  g_assert(item.type == MAP_TYPE_WAY);
+  g_assert(item.object.type == WAY);
 
-  osm_way_revert(item.way);
-  item.way->flags |= OSM_FLAG_DIRTY;
+  osm_way_reverse(item.object.way);
+  guint n_tags_flipped = 
+    osm_way_reverse_direction_sensitive_tags(item.object.way);
+  guint n_roles_flipped = 
+    osm_way_reverse_direction_sensitive_roles(appdata->osm, item.object.way);
 
-  map_way_select(appdata, item.way);
+  item.object.way->flags |= OSM_FLAG_DIRTY;
+  map_way_select(appdata, item.object.way);
+
+  // Flash a message about any side-effects
+  char *msg = NULL;
+  if (n_tags_flipped && !n_roles_flipped) {
+    msg = g_strdup_printf(ngettext("%d tag updated", "%d tags updated",
+                                   n_tags_flipped),
+                          n_tags_flipped);
+  }
+  else if (!n_tags_flipped && n_roles_flipped) {
+    msg = g_strdup_printf(ngettext("%d relation updated",
+                                   "%d relations updated",
+                                   n_roles_flipped),
+                          n_roles_flipped);
+  }
+  else if (n_tags_flipped && n_roles_flipped) {
+    char *msg1 = g_strdup_printf(ngettext("%d tag", "%d tags",
+                                          n_tags_flipped),
+                                 n_tags_flipped);
+    char *msg2 = g_strdup_printf(ngettext("%d relation", "%d relations",
+                                          n_roles_flipped),
+                                 n_roles_flipped);
+    msg = g_strdup_printf(_("%s & %s updated"), msg1, msg2);
+    g_free(msg1);
+    g_free(msg2);
+  }
+  if (msg) {
+    banner_show_info(appdata, msg);
+    g_free(msg);
+  }
 }
 
+// vim:et:ts=8:sw=2:sts=2:ai
