@@ -27,6 +27,8 @@
 #define COLOR_ERR  "red"
 #define COLOR_OK   "darkgreen"
 
+#undef NO_EXPECT
+
 static struct http_message_s {
   int id;
   char *msg;
@@ -238,8 +240,8 @@ static gboolean osm_update_item(struct log_s *log, char *xml_str,
     curl_easy_setopt(curl, CURLOPT_USERAGENT, 
 		     PACKAGE "-libcurl/" VERSION); 
     
-#ifndef API06
-    struct curl_slist *slist=NULL;
+#ifdef NO_EXPECT
+    struct curl_slist *slist = NULL;
     slist = curl_slist_append(slist, "Expect:");
     curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist);
 #endif
@@ -256,7 +258,7 @@ static gboolean osm_update_item(struct log_s *log, char *xml_str,
     curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response);
     
     /* always cleanup */
-#ifndef API06
+#ifdef NO_EXPECT
     curl_slist_free_all(slist);
 #endif
     curl_easy_cleanup(curl);
