@@ -20,6 +20,7 @@
 #include "appdata.h"
 
 /* UI sizes */
+/* TH: All this dialog size stuff should imho go into one central place */
 
 #ifdef USE_HILDON
 // Making the dialog a little wider makes it less "crowded" 
@@ -36,50 +37,13 @@ static const guint LIST_OF_MEMBERS_DIALOG_HEIGHT  = 350;
 #endif
 
 
-/* ------------------------- some generic treeview helpers -----------------------*/
-
-
-static void
-list_pre_inplace_edit_tweak (GtkTreeModel *model)
-{
-  // Remove any current sort ordering, leaving items where they are.
-  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model),
-                                       GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
-                                       GTK_SORT_ASCENDING);
-}
-
-
-/* Refocus a GtkTreeView an item specified by iter, unselecting the current
-   selection and optionally highlighting the new one. Typically called after
-   making an edit to an item with a covering sub-dialog. */
-
-static void
-list_focus_on (GtkTreeView *view, GtkTreeModel *model,
-               GtkTreeIter *iter, gboolean highlight)
-{
-  // Handle de/reselection
-  GtkTreeSelection *sel = gtk_tree_view_get_selection(view);
-  gtk_tree_selection_unselect_all(sel);
-
-  // Scroll to it, since it might now be out of view.
-  GtkTreePath *path = gtk_tree_model_get_path(model, iter);
-  gtk_tree_view_scroll_to_cell(view, path, NULL, FALSE, 0, 0);
-  gtk_tree_path_free(path);
-  
-  // reselect
-  if (highlight)
-    gtk_tree_selection_select_iter(sel, iter);
-}
-
-
 /* --------------- relation dialog for an item (node or way) ----------- */
 
 typedef struct {
-  relation_item_t *item;
+  object_t *item;
   appdata_t *appdata;
-  GtkWidget *dialog, *view;
+  GtkWidget *dialog, *list;
   GtkListStore *store;
-  GtkWidget *but_add, *but_edit, *but_remove;
 } relitem_context_t;
 
 enum {
@@ -97,13 +61,13 @@ typedef struct role_chain_s {
 } role_chain_t;
 
 static gboolean relation_add_item(GtkWidget *parent,
-			      relation_t *relation, relation_item_t *item) {
+			      relation_t *relation, object_t *object) {
   role_chain_t *chain = NULL, **chainP = &chain;
 
-  printf("add item of type %d to relation #%ld\n", 
-	 item->type, relation->id);
+  printf("add object of type %d to relation #" ITEM_ID_FORMAT "\n", 
+	 object->type, relation->id);
 
-  /* ask the user for the role of the new item in this relation */
+  /* ask the user for the role of the new object in this relation */
   
   /* collect roles first */
   member_t *member = relation->member;
@@ -140,7 +104,8 @@ static gboolean relation_add_item(GtkWidget *parent,
 
   char *info_str = NULL;
   if(type) info_str = g_strdup_printf(_("In relation of type: %s"), type);
-  else     info_str = g_strdup_printf(_("In relation #%ld"), relation->id);
+  else     info_str = g_strdup_printf(_("In relation #" ITEM_ID_FORMAT), 
+				      relation->id);
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
 			      gtk_label_new(info_str));
   g_free(info_str);
@@ -196,41 +161,32 @@ static gboolean relation_add_item(GtkWidget *parent,
   member_t **memberP = &relation->member;
   while(*memberP) memberP = &(*memberP)->next;
 
+  g_assert((object->type == NODE)||(object->type == WAY)||
+	   (object->type == RELATION));
+
   /* create new member */
   *memberP = g_new0(member_t, 1);
-  (*memberP)->type = item->type;
+  (*memberP)->object = *object;
   (*memberP)->role = role;
-  switch(item->type) {
-  case NODE:
-    (*memberP)->node = item->node;
-    break;
-  case WAY:
-    (*memberP)->way = item->way;
-    break;
-  case RELATION:
-    (*memberP)->relation = item->relation;
-    break;
-  default:
-    g_assert((item->type == NODE)||(item->type == WAY)||
-	     (item->type == RELATION));
-    break;
-  }
 
   relation->flags |= OSM_FLAG_DIRTY;
   return TRUE;
 }
 
-static void relation_remove_item(relation_t *relation, relation_item_t *item) {
+static void relation_remove_item(relation_t *relation, object_t *object) {
 
-  printf("remove item of type %d from relation #%ld\n", 
-	 item->type, relation->id);
+  printf("remove object of type %d from relation #" ITEM_ID_FORMAT "\n", 
+	 object->type, relation->id);
 
   member_t **member = &relation->member;
   while(*member) {
-    if(((*member)->type == item->type) &&
-       (((item->type == NODE) && (item->node == (*member)->node)) ||
-	((item->type == WAY) && (item->way == (*member)->way)) ||
-      ((item->type == RELATION) && (item->relation == (*member)->relation)))) {
+    if(((*member)->object.type == object->type) &&
+       (((object->type == NODE) && 
+	 (object->node == (*member)->object.node)) ||
+	((object->type == WAY) && 
+	 (object->way == (*member)->object.way)) ||
+	((object->type == RELATION) && 
+	 (object->relation == (*member)->object.relation)))) {
       
       member_t *next = (*member)->next;
       osm_member_free(*member);
@@ -248,25 +204,8 @@ static void relation_remove_item(relation_t *relation, relation_item_t *item) {
 static void relation_item_list_selected(relitem_context_t *context, 
 				   gboolean selected) {
 
-  if(context->but_remove)
-    gtk_widget_set_sensitive(context->but_remove, selected);
-  if(context->but_edit)
-    gtk_widget_set_sensitive(context->but_edit, selected);
-}
-
-static gboolean
-relation_item_list_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
-		     GtkTreePath *path, gboolean path_currently_selected,
-		     gpointer userdata) {
-  relitem_context_t *context = (relitem_context_t*)userdata;
-  GtkTreeIter iter;
-    
-  if(gtk_tree_model_get_iter(model, &iter, path)) {
-    g_assert(gtk_tree_path_get_depth(path) == 1);
-    relation_item_list_selected(context, TRUE);
-  }
-  
-  return TRUE; /* allow selection state to change */
+  list_button_enable(context->list, LIST_BUTTON_REMOVE, selected);
+  list_button_enable(context->list, LIST_BUTTON_EDIT, selected);
 }
 
 /* try to find something descriptive */
@@ -281,11 +220,19 @@ static char *relation_get_descriptive_name(relation_t *relation) {
   return name;
 }
 
+static gboolean relation_info_dialog(GtkWidget *parent, appdata_t *appdata, 
+				     relation_t *relation) {
+
+  object_t object = { .type = RELATION };
+  object.relation = relation;
+  return info_dialog(parent, appdata, &object);
+} 
+
 static void on_relation_item_add(GtkWidget *but, relitem_context_t *context) {
   /* create a new relation */
 
   relation_t *relation = osm_relation_new();
-  if(!info_dialog(context->dialog, context->appdata, relation)) {
+  if(!relation_info_dialog(context->dialog, context->appdata, relation)) {
     printf("tag edit cancelled\n");
     osm_relation_free(relation);
   } else {
@@ -306,12 +253,7 @@ static void on_relation_item_add(GtkWidget *but, relitem_context_t *context) {
 		       RELITEM_COL_DATA, relation,
 		       -1);
 
-    gtk_tree_selection_select_iter(gtk_tree_view_get_selection(
-	       GTK_TREE_VIEW(context->view)), &iter);
-
-    /* scroll to end */
-    //    GtkAdjustment *adj = gtk_scrolled_window_get_vadjustment();
-    /* xyz */
+    gtk_tree_selection_select_iter(list_get_selection(context->list), &iter);
   }
 }
 
@@ -320,7 +262,7 @@ static relation_t *get_selection(relitem_context_t *context) {
   GtkTreeModel     *model;
   GtkTreeIter       iter;
 
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view));
+  selection = list_get_selection(context->list);
   if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
     relation_t *relation;
     gtk_tree_model_get(model, &iter, RELITEM_COL_DATA, &relation, -1);
@@ -333,9 +275,9 @@ static void on_relation_item_edit(GtkWidget *but, relitem_context_t *context) {
   relation_t *sel = get_selection(context);
   if(!sel) return;
 
-  printf("edit relation item #%ld\n", sel->id);
+  printf("edit relation item #" ITEM_ID_FORMAT "\n", sel->id);
 
-  if (!info_dialog(context->dialog, context->appdata, sel))
+  if (!relation_info_dialog(context->dialog, context->appdata, sel))
     return;
 
   // Locate the changed item
@@ -361,8 +303,7 @@ static void on_relation_item_edit(GtkWidget *but, relitem_context_t *context) {
     -1);
 
   // Order will probably have changed, so refocus
-  list_focus_on(GTK_TREE_VIEW(context->view), GTK_TREE_MODEL(context->store),
-                &iter, TRUE);
+  list_focus_on(context->list, &iter, TRUE);
 }
 
 /* remove the selected relation */
@@ -370,7 +311,7 @@ static void on_relation_item_remove(GtkWidget *but, relitem_context_t *context)
   relation_t *sel = get_selection(context);
   if(!sel) return;
 
-  printf("remove relation #%ld\n", sel->id);
+  printf("remove relation #" ITEM_ID_FORMAT "\n", sel->id);
 
   gint members = osm_relation_members_num(sel);
 
@@ -383,8 +324,7 @@ static void on_relation_item_remove(GtkWidget *but, relitem_context_t *context)
   
   /* first remove selected row from list */
   GtkTreeIter       iter;
-  GtkTreeSelection *selection = 
-    gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view));
+  GtkTreeSelection *selection = list_get_selection(context->list);
   if(gtk_tree_selection_get_selected(selection, NULL, &iter)) 
     gtk_list_store_remove(context->store, &iter);
 
@@ -394,18 +334,18 @@ static void on_relation_item_remove(GtkWidget *but, relitem_context_t *context)
   relation_item_list_selected(context, FALSE);
 }
 
-static char *relitem_get_role_in_relation(relation_item_t *item, relation_t *relation) {
+static char *relitem_get_role_in_relation(object_t *item, relation_t *relation) {
   member_t *member = relation->member;
   while(member) {
-    switch(member->type) {
+    switch(member->object.type) {
 
     case NODE:
-      if((item->type == NODE) && (item->node == member->node))
+      if((item->type == NODE) && (item->node == member->object.node))
 	return member->role;
       break;
 
     case WAY:
-      if((item->type == WAY) && (item->way == member->way))
+      if((item->type == WAY) && (item->way == member->object.way))
 	return member->role;
       break;
 
@@ -456,18 +396,18 @@ relitem_toggled(GtkCellRendererToggle *cell, const gchar *path_str,
 
 }
 
-static gboolean relitem_is_in_relation(relation_item_t *item, relation_t *relation) {
+static gboolean relitem_is_in_relation(object_t *item, relation_t *relation) {
   member_t *member = relation->member;
   while(member) {
-    switch(member->type) {
+    switch(member->object.type) {
 
     case NODE:
-      if((item->type == NODE) && (item->node == member->node))
+      if((item->type == NODE) && (item->node == member->object.node))
 	return TRUE;
       break;
 
     case WAY:
-      if((item->type == WAY) && (item->way == member->way))
+      if((item->type == WAY) && (item->way == member->object.way))
 	return TRUE;
       break;
 
@@ -480,54 +420,22 @@ static gboolean relitem_is_in_relation(relation_item_t *item, relation_t *relati
 }
 
 static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
-  GtkWidget *vbox = gtk_vbox_new(FALSE,3);
-  context->view = gtk_tree_view_new();
-
-  gtk_tree_selection_set_select_function(
-	 gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view)), 
-	 relation_item_list_selection_func, 
-	 context, NULL);
-
-
-  /* --- "selected" column --- */
-  GtkCellRenderer *renderer = gtk_cell_renderer_toggle_new();
-  GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
-    _(""), renderer, "active", RELITEM_COL_SELECTED, NULL);
-  g_signal_connect(renderer, "toggled", G_CALLBACK(relitem_toggled), context);
-  gtk_tree_view_column_set_sort_column_id(column, RELITEM_COL_SELECTED);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);  
-
-  /* --- "Type" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Type"), renderer,
-    "text", RELITEM_COL_TYPE, NULL);
-  gtk_tree_view_column_set_sort_column_id(column, RELITEM_COL_TYPE);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);  
-
-  /* --- "Role" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Role"), renderer,
-    "text", RELITEM_COL_ROLE, NULL);
-  gtk_tree_view_column_set_sort_column_id(column, RELITEM_COL_ROLE);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);  
-
-  /* --- "Name" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-  column = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, 
-		 "text", RELITEM_COL_NAME, NULL);
-  gtk_tree_view_column_set_expand(column, TRUE);
-  gtk_tree_view_column_set_sort_column_id(column, RELITEM_COL_NAME);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
+  context->list = list_new(LIST_HILDON_WITH_HEADERS);
 
+  list_set_columns(context->list, 
+		   _(""), RELITEM_COL_SELECTED, LIST_FLAG_TOGGLE, 
+		            G_CALLBACK(relitem_toggled), context,
+		   _("Type"), RELITEM_COL_TYPE, 0,
+		   _("Role"), RELITEM_COL_ROLE, 0,
+		   _("Name"), RELITEM_COL_NAME, LIST_FLAG_ELLIPSIZE,
+		   NULL);
 
   /* build and fill the store */
   context->store = gtk_list_store_new(RELITEM_NUM_COLS, 
 		G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, 
 	        G_TYPE_STRING, G_TYPE_POINTER);
 
-  gtk_tree_view_set_model(GTK_TREE_VIEW(context->view), 
-			  GTK_TREE_MODEL(context->store));
+  list_set_store(context->list, context->store);
 
   // Debatable whether to sort by the "selected" or the "Name" column by
   // default. Both are be useful, in different ways.
@@ -555,60 +463,35 @@ static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
   
   g_object_unref(context->store);
 
-  /* put it into a scrolled window */
-  GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
-				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
-				      GTK_SHADOW_ETCHED_IN);
-  gtk_container_add(GTK_CONTAINER(scrolled_window), context->view);
-
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), scrolled_window);
-
-  /* ------- button box ------------ */
-
-  GtkWidget *hbox = gtk_hbox_new(TRUE,3);
-
-  context->but_add = gtk_button_new_with_label(_("Add..."));
-  //  gtk_widget_set_sensitive(context->but_add, FALSE);
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_add);
-  gtk_signal_connect(GTK_OBJECT(context->but_add), "clicked",
-    		     GTK_SIGNAL_FUNC(on_relation_item_add), context);
-
-  context->but_edit = gtk_button_new_with_label(_("Edit..."));
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_edit);
-  gtk_signal_connect(GTK_OBJECT(context->but_edit), "clicked",
-    		     GTK_SIGNAL_FUNC(on_relation_item_edit), context);
-
-  context->but_remove = gtk_button_new_with_label(_("Remove"));
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_remove);
-  gtk_signal_connect(GTK_OBJECT(context->but_remove), "clicked",
-  		     GTK_SIGNAL_FUNC(on_relation_item_remove), context);
+  list_set_static_buttons(context->list, G_CALLBACK(on_relation_item_add),
+	G_CALLBACK(on_relation_item_edit),G_CALLBACK(on_relation_item_remove), 
+        context);
 
   relation_item_list_selected(context, FALSE);
 
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-  return vbox;
+  return context->list;
 }
 
-void relation_add_dialog(appdata_t *appdata, relation_item_t *relitem) {
+void relation_add_dialog(appdata_t *appdata, object_t *object) {
   relitem_context_t *context = g_new0(relitem_context_t, 1);
   map_t *map = appdata->map;
   g_assert(map);
 
   context->appdata = appdata;
-  context->item = relitem;
+  context->item = object;
 
   char *str = NULL;
-  switch(relitem->type) {
+  switch(object->type) {
   case NODE:
-    str = g_strdup_printf(_("Relations for node #%ld"), relitem->node->id);
+    str = g_strdup_printf(_("Relations for node #" ITEM_ID_FORMAT), 
+			  object->node->id);
     break;
   case WAY:
-    str = g_strdup_printf(_("Relations for way #%ld"), relitem->way->id);
+    str = g_strdup_printf(_("Relations for way #" ITEM_ID_FORMAT), 
+			  object->way->id);
     break;
   default:
-    g_assert((relitem->type == NODE) || (relitem->type == WAY));
+    g_assert((object->type == NODE) || (object->type == WAY));
     break;
   }
   
@@ -640,8 +523,7 @@ void relation_add_dialog(appdata_t *appdata, relation_item_t *relitem) {
 
 typedef struct {
   appdata_t *appdata;
-  GtkWidget *dialog, *view;
-  GtkWidget *but_members, *but_add, *but_edit, *but_remove;
+  GtkWidget *dialog, *list, *show_btn;
   GtkListStore *store;
 } relation_context_t;
 
@@ -659,7 +541,7 @@ static relation_t *get_selected_relation(relation_context_t *context) {
   GtkTreeModel     *model;
   GtkTreeIter       iter;
 
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view));
+  selection = list_get_selection(context->list);
   if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
     relation_t *relation;
     gtk_tree_model_get(model, &iter, RELATION_COL_DATA, &relation, -1);
@@ -671,14 +553,13 @@ static relation_t *get_selected_relation(relation_context_t *context) {
 static void relation_list_selected(relation_context_t *context, 
 				   relation_t *selected) {
 
-  if(context->but_members)
-    gtk_widget_set_sensitive(context->but_members, 
-	     (selected != NULL) && (selected->member != NULL));
+  list_button_enable(context->list, LIST_BUTTON_USER0, 
+		     (selected != NULL) && (selected->member != NULL));
+  gtk_widget_set_sensitive(context->show_btn,
+		     (selected != NULL) && (selected->member != NULL));
 
-  if(context->but_remove)
-    gtk_widget_set_sensitive(context->but_remove, selected != NULL);
-  if(context->but_edit)
-    gtk_widget_set_sensitive(context->but_edit, selected != NULL);
+  list_button_enable(context->list, LIST_BUTTON_REMOVE, selected != NULL);
+  list_button_enable(context->list, LIST_BUTTON_EDIT, selected != NULL);
 }
 
 static gboolean
@@ -726,7 +607,7 @@ member_list_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
 
     member_t *member = NULL;
     gtk_tree_model_get(model, &iter, MEMBER_COL_DATA, &member, -1);
-    if(member && member->type < NODE_ID)
+    if(member && member->object.type < NODE_ID)
       return TRUE;
   }
   
@@ -746,35 +627,43 @@ static GtkWidget *member_list_widget(member_context_t *context) {
   /* --- "type" column --- */
   GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
   g_object_set(renderer, "foreground", "grey", NULL);
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(context->view),
-	-1, _("Type"), renderer, "text", MEMBER_COL_TYPE,
-        "foreground-set", MEMBER_COL_REF_ONLY,  NULL); 
-
+  GtkTreeViewColumn *column = 
+    gtk_tree_view_column_new_with_attributes(_("Type"), renderer, 
+		     "text", MEMBER_COL_TYPE,
+		     "foreground-set", MEMBER_COL_REF_ONLY,  NULL);
+  gtk_tree_view_column_set_sort_column_id(column, MEMBER_COL_TYPE); 
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
 
   /* --- "id" column --- */
   renderer = gtk_cell_renderer_text_new();
   g_object_set(renderer, "foreground", "grey", NULL);
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(context->view),
-	-1, _("Id"), renderer, "text", MEMBER_COL_ID,
-        "foreground-set", MEMBER_COL_REF_ONLY, NULL); 
+  column = gtk_tree_view_column_new_with_attributes(_("Id"), renderer, 
+		     "text", MEMBER_COL_ID,
+		     "foreground-set", MEMBER_COL_REF_ONLY,  NULL);
+  gtk_tree_view_column_set_sort_column_id(column, MEMBER_COL_ID); 
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
+
 
   /* --- "Name" column --- */
   renderer = gtk_cell_renderer_text_new();
   g_object_set(renderer, "foreground", "grey", NULL);
   g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-  GtkTreeViewColumn *column = 
-    gtk_tree_view_column_new_with_attributes(_("Name"), renderer, 
+  column = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, 
 		     "text", MEMBER_COL_NAME,
 		     "foreground-set", MEMBER_COL_REF_ONLY,  NULL);
   gtk_tree_view_column_set_expand(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, MEMBER_COL_NAME); 
   gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
 
   /* --- "role" column --- */
   renderer = gtk_cell_renderer_text_new();
   g_object_set(renderer, "foreground", "grey", NULL);
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(context->view),
-	-1, _("Role"), renderer, "text", MEMBER_COL_ROLE,
-        "foreground-set", MEMBER_COL_REF_ONLY, NULL); 
+  column = gtk_tree_view_column_new_with_attributes(_("Role"), renderer, 
+		     "text", MEMBER_COL_ROLE,
+		     "foreground-set", MEMBER_COL_REF_ONLY,  NULL);
+  gtk_tree_view_column_set_sort_column_id(column, MEMBER_COL_ROLE); 
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
+
 
   /* build and fill the store */
   context->store = gtk_list_store_new(MEMBER_NUM_COLS, 
@@ -787,8 +676,8 @@ static GtkWidget *member_list_widget(member_context_t *context) {
   GtkTreeIter iter;
   member_t *member = context->relation->member;
   while(member) {
-    tag_t *tags = osm_object_get_tags(member->type, member->ptr);
-    char *id = osm_id_string(member->type, member->ptr);
+    tag_t *tags = osm_object_get_tags(&member->object);
+    char *id = osm_object_id_string(&member->object);
 
     /* try to find something descriptive */
     char *name = NULL;
@@ -798,11 +687,11 @@ static GtkWidget *member_list_widget(member_context_t *context) {
     /* Append a row and fill in some data */
     gtk_list_store_append(context->store, &iter);
     gtk_list_store_set(context->store, &iter,
-       MEMBER_COL_TYPE, osm_type_string(member->type),
+       MEMBER_COL_TYPE, osm_object_type_string(&member->object),
        MEMBER_COL_ID,   id,
        MEMBER_COL_NAME, name, 
        MEMBER_COL_ROLE, member->role,
-       MEMBER_COL_REF_ONLY, member->type >= NODE_ID,
+       MEMBER_COL_REF_ONLY, member->object.type >= NODE_ID,
        MEMBER_COL_DATA, member,
        -1);
 
@@ -825,25 +714,21 @@ static GtkWidget *member_list_widget(member_context_t *context) {
   return vbox;
 }
 
-/* user clicked "members..." button in relation list */
-static void on_relation_members(GtkWidget *but, relation_context_t *context) {
+void relation_show_members(GtkWidget *parent, relation_t *relation) {
   member_context_t *mcontext = g_new0(member_context_t, 1);
-
-  /* display members list */
-  mcontext->relation = get_selected_relation(context);
-  if(!mcontext->relation) return;
+  mcontext->relation = relation;
 
   char *str = osm_tag_get_by_key(mcontext->relation->tag, "name");
   if(!str) str = osm_tag_get_by_key(mcontext->relation->tag, "ref");
   if(!str)
-    str = g_strdup_printf(_("Members of relation #%ld"),
+    str = g_strdup_printf(_("Members of relation #" ITEM_ID_FORMAT),
 			  mcontext->relation->id);
   else
     str = g_strdup_printf(_("Members of relation \"%s\""), str);
 
   mcontext->dialog = 
     gtk_dialog_new_with_buttons(str,
-	GTK_WINDOW(context->dialog), GTK_DIALOG_MODAL,
+	GTK_WINDOW(parent), GTK_DIALOG_MODAL,
 	GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, 
 	NULL);
   g_free(str);
@@ -867,11 +752,20 @@ static void on_relation_members(GtkWidget *but, relation_context_t *context) {
   g_free(mcontext);
 }
 
+/* user clicked "members..." button in relation list */
+static void on_relation_members(GtkWidget *but, relation_context_t *context) {
+  relation_t *sel = get_selected_relation(context);
+  
+  if(sel)
+    relation_show_members(context->dialog, sel);
+}
+
+
 static void on_relation_add(GtkWidget *but, relation_context_t *context) {
   /* create a new relation */
 
   relation_t *relation = osm_relation_new();
-  if(!info_dialog(context->dialog, context->appdata, relation)) {
+  if(!relation_info_dialog(context->dialog, context->appdata, relation)) {
     printf("tag edit cancelled\n");
     osm_relation_free(relation);
   } else {
@@ -895,8 +789,7 @@ static void on_relation_add(GtkWidget *but, relation_context_t *context) {
 		       RELATION_COL_DATA, relation,
 		       -1);
 
-    gtk_tree_selection_select_iter(gtk_tree_view_get_selection(
-	       GTK_TREE_VIEW(context->view)), &iter);
+    gtk_tree_selection_select_iter(list_get_selection(context->list), &iter);
 
     /* scroll to end */
     //    GtkAdjustment *adj = gtk_scrolled_window_get_vadjustment();
@@ -909,9 +802,9 @@ static void on_relation_edit(GtkWidget *but, relation_context_t *context) {
   relation_t *sel = get_selected_relation(context);
   if(!sel) return;
 
-  printf("edit relation #%ld\n", sel->id);
+  printf("edit relation #" ITEM_ID_FORMAT "\n", sel->id);
 
-  if (!info_dialog(context->dialog, context->appdata, sel))
+  if (!relation_info_dialog(context->dialog, context->appdata, sel))
     return;
 
   // Locate the changed item
@@ -939,8 +832,7 @@ static void on_relation_edit(GtkWidget *but, relation_context_t *context) {
     -1);
 
   // Order will probably have changed, so refocus
-  list_focus_on(GTK_TREE_VIEW(context->view), GTK_TREE_MODEL(context->store),
-                &iter, TRUE);
+  list_focus_on(context->list, &iter, TRUE);
 }
 
 
@@ -949,7 +841,7 @@ static void on_relation_remove(GtkWidget *but, relation_context_t *context) {
   relation_t *sel = get_selected_relation(context);
   if(!sel) return;
 
-  printf("remove relation #%ld\n", sel->id);
+  printf("remove relation #" ITEM_ID_FORMAT "\n", sel->id);
 
   gint members = osm_relation_members_num(sel);
 
@@ -962,8 +854,7 @@ static void on_relation_remove(GtkWidget *but, relation_context_t *context) {
   
   /* first remove selected row from list */
   GtkTreeIter       iter;
-  GtkTreeSelection *selection = 
-    gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view));
+  GtkTreeSelection *selection = list_get_selection(context->list);
   if(gtk_tree_selection_get_selected(selection, NULL, &iter)) 
     gtk_list_store_remove(context->store, &iter);
 
@@ -974,51 +865,24 @@ static void on_relation_remove(GtkWidget *but, relation_context_t *context) {
 }
 
 static GtkWidget *relation_list_widget(relation_context_t *context) {
-  GtkWidget *vbox = gtk_vbox_new(FALSE,3);
-  context->view = gtk_tree_view_new();
+  context->list = list_new(LIST_HILDON_WITH_HEADERS);
 
-  gtk_tree_selection_set_select_function(
-	 gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view)), 
-	 relation_list_selection_func, 
-	 context, NULL);
+  list_set_selection_function(context->list, relation_list_selection_func, 
+			      context);
 
-  /* --- "id" column --- */
-  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
-  GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
-    _("Id"), renderer, "text", RELATION_COL_ID, NULL);
-  gtk_tree_view_column_set_sort_column_id(column, RELATION_COL_ID);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
-
-  /* --- "Type" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Type"), renderer,
-    "text", RELATION_COL_TYPE, NULL);
-  gtk_tree_view_column_set_sort_column_id(column, RELATION_COL_TYPE);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
-
-  /* --- "Name" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-  column = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, 
-		 "text", RELATION_COL_NAME, NULL);
-  gtk_tree_view_column_set_expand(column, TRUE);
-  gtk_tree_view_column_set_sort_column_id(column, RELATION_COL_NAME);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
-
-  /* --- "members" column --- */
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Members"), renderer, 
-		 "text", RELATION_COL_MEMBERS, NULL);
-  gtk_tree_view_column_set_sort_column_id(column, RELATION_COL_MEMBERS);
-  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
+  list_set_columns(context->list,
+		   _("Id"),      RELATION_COL_ID, 0,
+		   _("Type"),    RELATION_COL_TYPE, 0,
+		   _("Name"),    RELATION_COL_NAME, LIST_FLAG_ELLIPSIZE,
+		   _("Members"), RELATION_COL_MEMBERS, 0,
+		   NULL);
 
   /* build and fill the store */
   context->store = gtk_list_store_new(RELATION_NUM_COLS, 
 		G_TYPE_ITEM_ID_T, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, 
 	        G_TYPE_POINTER);
 
-  gtk_tree_view_set_model(GTK_TREE_VIEW(context->view), 
-			  GTK_TREE_MODEL(context->store));
+  list_set_store(context->list, context->store);
 
   // Sorting by ref/name by default is useful for places with lots of numbered
   // bus routes. Especially for small screens.
@@ -1048,47 +912,16 @@ static GtkWidget *relation_list_widget(relation_context_t *context) {
   
   g_object_unref(context->store);
 
-  /* put it into a scrolled window */
-  GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
-				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
-				      GTK_SHADOW_ETCHED_IN);
-  gtk_container_add(GTK_CONTAINER(scrolled_window), context->view);
+  list_set_static_buttons(context->list, G_CALLBACK(on_relation_add),
+     G_CALLBACK(on_relation_edit), G_CALLBACK(on_relation_remove), context);
 
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), scrolled_window);
-
-  /* ------- button box ------------ */
-
-  GtkWidget *hbox = gtk_hbox_new(TRUE,3);
-
-  context->but_add = gtk_button_new_with_label(_("Add..."));
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_add);
-  gtk_signal_connect(GTK_OBJECT(context->but_add), "clicked",
-      		     GTK_SIGNAL_FUNC(on_relation_add), context);
-
-  context->but_edit = gtk_button_new_with_label(_("Edit..."));
-  gtk_widget_set_sensitive(context->but_edit, FALSE);
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_edit);
-  gtk_signal_connect(GTK_OBJECT(context->but_edit), "clicked",
-      		     GTK_SIGNAL_FUNC(on_relation_edit), context);
-
-  context->but_members = gtk_button_new_with_label(_("Members..."));
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_members);
-  gtk_signal_connect(GTK_OBJECT(context->but_members), "clicked",
-  		     GTK_SIGNAL_FUNC(on_relation_members), context);
-
-  context->but_remove = gtk_button_new_with_label(_("Remove"));
-  gtk_widget_set_sensitive(context->but_remove, FALSE);
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_remove);
-  gtk_signal_connect(GTK_OBJECT(context->but_remove), "clicked",
-    		     GTK_SIGNAL_FUNC(on_relation_remove), context);
-
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  list_set_user_buttons(context->list, 
+	LIST_BUTTON_USER0, _("Members..."), G_CALLBACK(on_relation_members),
+	0);
 
   relation_list_selected(context, NULL);
 
-  return vbox;
+  return context->list;
 }
 
 /* a global view on all relations */
@@ -1111,15 +944,24 @@ void relation_list(appdata_t *appdata) {
                               LIST_OF_RELATIONS_DIALOG_WIDTH,
                               LIST_OF_RELATIONS_DIALOG_HEIGHT);
   
+  context->show_btn = gtk_dialog_add_button(GTK_DIALOG(context->dialog), 
+					    _("Select"), GTK_RESPONSE_HELP);
+
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(context->dialog)->vbox),
   		     relation_list_widget(context), TRUE, TRUE, 0);
 
   /* ----------------------------------- */
 
-  gtk_widget_show_all(context->dialog);
-  gtk_dialog_run(GTK_DIALOG(context->dialog));
-  gtk_widget_destroy(context->dialog);
 
+  gtk_widget_show_all(context->dialog);
+  if(gtk_dialog_run(GTK_DIALOG(context->dialog)) == GTK_RESPONSE_HELP) {
+    map_item_deselect(appdata);
+
+    relation_t *sel = get_selected_relation(context);
+    if(sel) map_relation_select(appdata, sel);
+  }
+
+  gtk_widget_destroy(context->dialog);
   g_free(context);
 }
 
